#!/usr/bin/env python
#Tiago de Freitas Pereira <tiagofrepereira@gmail.com>
#Fri Jul 13 14:30:00 CEST 2012

"""Calculates the LBPTop planes (XY,XT,YT) of the normalized faces in the videos in the REPLAY-ATTACK database. This code extract the LBP-TOP features using the MULTIRESOLUTION approach, setting more than one value for Rt. The result is the LBP histogram over all orthogonal frames of the video (XY,XT,YT). Different types of LBP operators are supported. The histograms can be computed for a subset of the videos in the database (using the protocols in REPLAY-ATTACK). The output is a single .hdf5 file for each video. The procedure is described in the paper: "LBP-TOP based countermeasure against facial spoofing attacks" - de Freitas Pereira, Tiago and Anjos, Andre and De Martino, Jose Mario and Marcel, Sebastien; ACCV - LBP 2012

"""

import os, sys
import argparse
import bob
import numpy

from .. import spoof
import antispoofing.utils
from antispoofing.utils.faceloc import *
from antispoofing.utils.db import *

def main():

  basedir = os.path.dirname(os.path.dirname(os.path.realpath(sys.argv[0])))

  INPUT_DIR = os.path.join(basedir, 'database')
  OUTPUT_DIR = os.path.join(basedir, 'dog_features')


  ##########
  # General configuration
  ##########

  parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
  parser.add_argument('-i', '--input-dir', metavar='DIR', type=str, dest='inputDir', default=INPUT_DIR, help='Base directory containing the videos to be treated by this procedure (defaults to "%(default)s")')

  parser.add_argument('-d', '--output-dir', dest="outputDir", default=OUTPUT_DIR, help="This path will be prepended to every file output by this procedure (defaults to '%(default)s')")

  parser.add_argument('-n', '--normface-size', dest="normfacesize", default=32, type=int, help="this is the size of the normalized face box if face normalization is used (defaults to '%(default)s')")

  parser.add_argument('--ff', '--facesize_filter', dest="facesize_filter", default=50, type=int, help="all the frames with faces smaller then this number, will be discarded (defaults to '%(default)s')")

  parser.add_argument('--s1', '--sigma-1', dest="sigma1", default=1, type=float, help="A set of Sigmas of the first gaussian",nargs='+')

  parser.add_argument('--s2', '--sigma-2', dest="sigma2", default=2, type=float, help="A set of Sigmas of the second gaussian",nargs='+')

  parser.add_argument('-r', '--radius', dest="radius", default=1, type=int, help="The Gaussian radius. Here, 1 means a 3x3 kernel.")

  parser.add_argument('-v', '--verbose', action='store_true', dest='verbose', default=False, help='Increases this script verbosity')

  # For SGE grid processing @ Idiap
  parser.add_argument('--grid', dest='grid', action='store_true', default=False, help=argparse.SUPPRESS)

  #######
  # Database especific configuration
  #######
  #Database.create_parser(parser, implements_any_of='video')
  Database.create_parser(parser)

  args = parser.parse_args()

  inputDir  = args.inputDir
  outputDir = args.outputDir

  normfacesize    = args.normfacesize
  facesize_filter = args.facesize_filter

  sigma1 = args.sigma1
  sigma2 = args.sigma2

  radius = args.radius

  verbose = args.verbose

  if not os.path.exists(inputDir):
    parser.error("input directory does not exist")

  if(len(sigma1) <> len(sigma2)):
    parser.error("The number of sigma-1 and sigma-2 must be the same")

  numberOfFilters = len(sigma1)
  flatImageSize   = normfacesize*normfacesize
  dimension       = flatImageSize*numberOfFilters

  ########################
  #Querying the database
  ########################
  #database = new_database(databaseName,args=args)
  database = args.cls(args)
  realObjects, attackObjects = database.get_all_data()
  process = realObjects + attackObjects 
  

  # finally, if we are on a grid environment, just find what I have to process.
  if args.grid:
    key = int(os.environ['SGE_TASK_ID']) - 1
    if key >= len(process):
      raise RuntimeError, "Grid request for job %d on a setup with %d jobs" % \
          (key, len(process))
    process = [process[key]]

  # processindatag each video
  for index, obj in enumerate(process):

    #Loading the file
    filename = str(obj.videofile(inputDir))

    #Loading the video
    input = bob.io.VideoReader(filename)

    #Loading the face locations
    flocfile = obj.facefile(inputDir)
    locations = preprocess_detections(flocfile,input.number_of_frames,facesize_filter=facesize_filter)

    if(verbose):
      sys.stdout.write("Processing file %s (%d frames) [%d/%d] " % (filename,
        input.number_of_frames, index+1, len(process)))
      sys.stdout.flush()

    # start the work here...
    vin = input.load() # load the video
    nFrames = vin.shape[0]

    #The amount of data to storage
    data = numpy.zeros((nFrames,dimension),dtype=float)

    #Converting all frames to grayscale
    grayFrames = numpy.zeros(shape=(nFrames,vin.shape[2],vin.shape[3]))

    for i in range(nFrames):
      
      #The amount of data for one frame
      frameData = numpy.zeros(dimension,dtype=float)

      #To gray
      grayFrames[i] = bob.ip.rgb_to_gray(vin[i,:,:,:])
      bbx = locations[i]

      if(bbx==None):
        data[i,:] = numpy.nan
        continue

      #Normalizing the face to a specific face size
      normFace = spoof.normalizeFace(grayFrames[i],bbx,normfacesize,facesize_filter)

      offsetFrame = 0
      #Applying each filter to a specific frame
      for j in range(numberOfFilters):
        s1 = sigma1[j]
        s2 = sigma2[j]

        #Calculating the DoG filter
        f = spoof.calcDoG(normFace,s1,s2,radius)
        frameData[offsetFrame:offsetFrame+flatImageSize] = numpy.reshape(f,flatImageSize)
        offsetFrame = offsetFrame + flatImageSize
 
      data[i,:] = frameData
      del frameData

    sys.stdout.write('\n')
    sys.stdout.flush()

    # saves the output
    obj.save(data,directory=outputDir,extension='.hdf5')
    del data

  return 0

if __name__ == "__main__":
  main()
